Package: simona
Type: Package
Title: Semantic Similarity on Bio-Ontologies
Version: 1.10.0
Date: 2026-01-30
Authors@R: person("Zuguang", "Gu", email = "guzuguang@suat-sz.edu.cn", role = c("aut", "cre"),
                  comment = c('ORCID'="0000-0002-7395-8709"))
Depends: R (>= 4.1.0)
Imports: methods, Rcpp, matrixStats, GetoptLong, grid, GlobalOptions,
        igraph, Polychrome, S4Vectors, xml2 (>= 1.3.3), circlize,
        ComplexHeatmap, grDevices, stats, utils, shiny, fastmatch
Suggests: knitr, testthat, BiocManager, GO.db, org.Hs.eg.db, proxyC,
        AnnotationDbi, Matrix, DiagrammeR, ragg, png,
        InteractiveComplexHeatmap, UniProtKeywords, simplifyEnrichment,
        AnnotationHub, jsonlite
LinkingTo: Rcpp
VignetteBuilder: knitr
Description: This package implements infrastructures for ontology analysis by offering 
    efficient data structures, fast ontology traversal methods, and elegant visualizations. 
    It provides a robust toolbox supporting over 70 methods for semantic similarity analysis.
biocViews: Software, Annotation, GO, BiomedicalInformatics
URL: https://github.com/jokergoo/simona
BugReports: https://github.com/jokergoo/simona/issues
SystemRequirements: Perl, Java
License: MIT + file LICENSE
RoxygenNote: 7.3.2
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/simona
git_branch: RELEASE_3_23
git_last_commit: 3c729fd
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:44:33 UTC; biocbuild
Author: Zuguang Gu [aut, cre] (ORCID: <https://orcid.org/0000-0002-7395-8709>)
Maintainer: Zuguang Gu <guzuguang@suat-sz.edu.cn>
