% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_global.R
\name{all_term_IC_methods}
\alias{all_term_IC_methods}
\alias{all_term_sim_methods}
\alias{all_group_sim_methods}
\title{Supported methods}
\usage{
all_term_IC_methods(require_anno = NULL)

all_term_sim_methods(require_anno = NULL)

all_group_sim_methods(require_anno = NULL)
}
\arguments{
\item{require_anno}{If it is set to \code{TRUE}, methods that require external annotations are only returned. If
it is set to \code{FALSE}, methods that do not require annotations are returned. A value of \code{NULL} returns both.}
}
\value{
A character vector of all supported methods.
}
\description{
Supported methods
}
\details{
\itemize{
\item \code{all_term_IC_methods()}: A vector of all supported IC methods.
\item \code{all_term_sim_methods()}: A vector of all supported term similarity methods.
\item \code{all_group_sim_methods()}: A vector of all supported group similarity methods.
}
}
\examples{
all_term_IC_methods()
all_term_sim_methods()
all_group_sim_methods()
}
